﻿using DocumentFormat.OpenXml.Office2010.Excel;
using Exam.Application;
using Exam.Application.Interface.Questions;
using Exam.ExamManages;
using Exam.Infrastructure.Data.Entity;
using Exam.Infrastructure.Enums;
using Exam.Infrastructure.Extensions;
using Exam.Infrastructure.Web.Extensions;
using Exam.Infrastructure.Web.Utilities;
using Exam.Insfrastructure.Service.Service;
using Exam.Questions;
using Exam.Repository.Sqlsugar.Repositories;
using Exam.Share.ViewResponses.Question;
using Hotline.Application.Exam.QueryExtensions.Questions;
using Hotline.Exams.Sourcewares;
using Hotline.Repository.SqlSugar;
using Hotline.Repository.SqlSugar.DataPermissions;
using Hotline.Repository.SqlSugar.Exam.Interfaces.Questions;
using Hotline.Repository.SqlSugar.Extensions;
using Hotline.Share.Dtos.Questions;
using Hotline.Share.Requests.Question;
using JiebaNet.Segmenter.Common;
using MapsterMapper;
using SqlSugar;
using XF.Domain.Dependency;
using XF.Domain.Entities;
using Hotline.Application.Exam.Extensions;
using Hotline.Application.Exam.Core.Extensions;
using Hotline.Application.Exam.Constants.Messages;
using Hotline.Repository.SqlSugar.Exam.Core.Constants;
using NPOI.SS.Formula.Functions;

namespace Hotline.Application.Exam.Service.Questions
{
    public class QuestionService : ApiService<Question, AddQuestionDto,UpdateQuestionDto, HotlineDbContext>, IQuestionService, IScopeDependency
    {
        #region ctor
        private readonly IQuestionRepository _repository;
        private readonly IQuestionTagRepository _questionTagRepository;
        private readonly IQuestionOptionsRepository _questionOptionRepository;
        private readonly IQuestionAnswerRepository _questionAnswerRepository;
        private readonly IQuestionSourcewareRepository _questionSourcewareRepository;
        private readonly IQuestionKnowladgeRepository _questionKnowladgeRepository;
        private readonly IDataPermissionFilterBuilder _dataPermissionFilterBuilder;
        private readonly IServiceProvider _serviceProvider;
        private readonly IMapper _mapper;

        public QuestionService(IQuestionRepository repository,
            IQuestionTagRepository questionTagRepository,
            IQuestionOptionsRepository questionOptionsRepository,
            IQuestionAnswerRepository questionAnswerRepository,
            IQuestionSourcewareRepository questionSourcewareRepository,
            IQuestionKnowladgeRepository questionKnowladgeRepository,
            IDataPermissionFilterBuilder dataPermissionFilterBuilder, IServiceProvider serviceProvider,
            IMapper mapper
            ) : base(repository,mapper)
        {
            _repository = repository;
            _questionTagRepository = questionTagRepository;
            _questionAnswerRepository = questionAnswerRepository;
            _questionOptionRepository = questionOptionsRepository;
            _questionSourcewareRepository = questionSourcewareRepository;
            _questionKnowladgeRepository = questionKnowladgeRepository;
            _dataPermissionFilterBuilder = dataPermissionFilterBuilder;
            _serviceProvider = serviceProvider;
            _mapper = mapper;
        }
        #endregion

        #region public method
        public async Task<QuestionDto> GetAsync(EntityQueryRequest entityQueryRequest)
        {
            var entity = await _repository.GetAsync(entityQueryRequest.Id);

            var questionDto = _mapper.Map<QuestionDto>(entity);

            if (questionDto != null)
            {
                questionDto.QuestionTagDtos = await GetQuestionTags(entityQueryRequest);

                questionDto.QuestionAnswerDto = await GetQuestionAnswer(entityQueryRequest);

                questionDto.QuestionKnowladgeDtos = await GetKnowladges(entityQueryRequest);

                questionDto.QuestionOptionsDtos = await GetQuestionOptions(entityQueryRequest);

                questionDto.QuestionSourcewareDtos = await GetQuestionSourcewares(entityQueryRequest);
            }            

            return questionDto;
        }

        public async Task<(int, List<QuestionViewResponse>)> GetListAsync(QuestionPagedRequest queryRequest)
        {
            var expression = queryRequest.GetExpression();
            var questionTable = _repository.Queryable().Where(expression);

            var questionKnowladgeExpression = queryRequest.GetQuestionKnowladgeExpression();
            var questionKnowladgeTable = new ExamRepository<QuestionKnowladge>(_repository.UOW, _dataPermissionFilterBuilder, _serviceProvider).Queryable().Where(questionKnowladgeExpression);

            var queryable = questionTable.InnerJoin(questionKnowladgeTable, (q, k) => q.Id == k.QuestionId).Select((q, k) => new QuestionViewResponse
            {
                DifficultyLevel = q.DifficultyLevel,
                Title = q.Title,
                QuestionType = q.QuestionType,
                Id= q.Id
            });

            var result = await queryable.ToListAsync();

            var total = await queryable.CountAsync();

            return (total, result);
        }

        public async Task<PageViewResponse<QuestionViewResponse>> GetPagedListAsync(QuestionPagedRequest queryRequest)
        {
            var expression = queryRequest.GetExpression();
            var questionTable = _repository.Queryable().Where(expression);

            var questionTagExpression = queryRequest.GetQuestionTagExpression();
            var questionTagTable = new BaseRepository<QuestionTag>(_repository.UOW, _dataPermissionFilterBuilder, _serviceProvider).Queryable().Where(questionTagExpression);

            var examTagExpression = queryRequest.GetExamTagExpression();
            var examTagTable = new BaseRepository<ExamTag>(_repository.UOW, _dataPermissionFilterBuilder, _serviceProvider).Queryable().Where(examTagExpression);


            var queryable = questionTable.InnerJoin(questionTagTable, (s, d) => s.Id == d.QuestionId).InnerJoin(examTagTable, (s,d, t) => d.TagId == t.Id).Select((s, d, t) => new QuestionViewResponse
            {
                DifficultyLevel = s.DifficultyLevel,
                FormalEnable = s.FormalEnable,
                SimulateEnable = s.SimulateEnable,
                SortIndex = s.SortIndex,
                Status = s.Status,
                Tag = t.Name,
                QuestionType = s.QuestionType,
                Title = s.Title,
                Id = s.Id
            });

            var list = await queryable.ToPageListAsync(queryRequest.PageIndex, queryRequest.PageSize);
            var total = await queryable.CountAsync();

            var result = new QuestionPageViewResponse
            {
                Items = list,
                Pagination = new Pagination(queryRequest.PageIndex, queryRequest.PageSize, total)
            };

            return result;
        }

        public override async Task<string> AddAsync(AddQuestionDto actionRequest, CancellationToken cancellationToken)
        {
            base.StartTran();

            var id = await base.AddAsync(actionRequest, cancellationToken);

            ResolveQuestionId(actionRequest,id);

            base.Entity.QuestionTags = await AddQuestionTags(actionRequest, cancellationToken);

            base.Entity.QuestionOptionses = await AddQuestionOptions(actionRequest, cancellationToken);

            base.Entity.QuestionAnswerE = await AddQuestionAnswer(actionRequest, cancellationToken);

            base.Entity.QuestionKnowladges =  await AddKnowladges(actionRequest, cancellationToken);

            base.Entity.QuestionSourcewares = await AddSourcewares(actionRequest, cancellationToken);

            await base.Complete(base.Entity, OperationConstant.Create);           

            return id;
        }

        public override async Task UpdateAsync(UpdateQuestionDto actionRequest, CancellationToken cancellationToken)
        {
            base.StartTran();

            await base.UpdateAsync(actionRequest, cancellationToken);

            ResolveQuestionId(actionRequest, actionRequest.Id);

            base.Entity.QuestionTags =  await ModifyQuestionTags(actionRequest, cancellationToken);

            base.Entity.QuestionOptionses =  await ModifyQuestionOptions(actionRequest, cancellationToken);

            base.Entity.QuestionAnswerE = await ModifyQuestionAnswer(actionRequest, cancellationToken);

            base.Entity.QuestionKnowladges =  await ModifyKnowladges(actionRequest, cancellationToken);

            base.Entity.QuestionSourcewares =  await ModifySourcewares(actionRequest, cancellationToken);

           await base.Complete(base.Entity,OperationConstant.Update);
        }

        public override async Task DeleteAsync(EntityQueryRequest entityQueryRequest, CancellationToken cancellationToken)
        {
            await base.DeleteAsync(entityQueryRequest, cancellationToken);

            var tmpEntityQueryRequest = ExpressionableUtility.CreateExpression<QuestionTag>()
                .AndIF(entityQueryRequest.Id.IsNotNullOrEmpty(), x => x.QuestionId == entityQueryRequest.Id)
                .AndIF(entityQueryRequest.Ids.IsNotNull(), x => entityQueryRequest.Ids.Contains(x.QuestionId))
                .ToEntityQueryRequest<QuestionTag>();

            await DeleteQuestionTags(tmpEntityQueryRequest, cancellationToken);

            tmpEntityQueryRequest = ExpressionableUtility.CreateExpression<QuestionOptions>()
                .AndIF(entityQueryRequest.Id.IsNotNullOrEmpty(), x => x.QuestionId == entityQueryRequest.Id)
                .AndIF(entityQueryRequest.Ids.IsNotNull(), x => entityQueryRequest.Ids.Contains(x.QuestionId))
                .ToEntityQueryRequest<QuestionOptions>();

            await DeleteQuestionOptions(tmpEntityQueryRequest, cancellationToken);

            tmpEntityQueryRequest = ExpressionableUtility.CreateExpression<QuestionAnswer>()
                .AndIF(entityQueryRequest.Id.IsNotNullOrEmpty(), x => x.QuestionId == entityQueryRequest.Id)
                .AndIF(entityQueryRequest.Ids.IsNotNull(), x => entityQueryRequest.Ids.Contains(x.QuestionId))
                .ToEntityQueryRequest<QuestionAnswer>();

            await DeleteQuestionAnswer(tmpEntityQueryRequest, cancellationToken);

            tmpEntityQueryRequest = ExpressionableUtility.CreateExpression<QuestionKnowladge>()
                .AndIF(entityQueryRequest.Id.IsNotNullOrEmpty(), x => x.QuestionId == entityQueryRequest.Id)
                .AndIF(entityQueryRequest.Ids.IsNotNull(), x => entityQueryRequest.Ids.Contains(x.QuestionId))
                .ToEntityQueryRequest<QuestionKnowladge>();

            await DeleteKnowladges(tmpEntityQueryRequest, cancellationToken);

            tmpEntityQueryRequest = ExpressionableUtility.CreateExpression<QuestionSourceware>()
                .AndIF(entityQueryRequest.Id.IsNotNullOrEmpty(), x => x.QuestionId == entityQueryRequest.Id)
                .AndIF(entityQueryRequest.Ids.IsNotNull(), x => entityQueryRequest.Ids.Contains(x.QuestionId))
                .ToEntityQueryRequest<QuestionSourceware>();

            await DeleteSourcewares(tmpEntityQueryRequest, cancellationToken);
        }
        #endregion

        #region private method

        private void ResolveQuestionId(AddQuestionDto actionRequest,string id) 
        {
            actionRequest.QuestionKnowladgeDtos?.ForEach(x => x.QuestionId = id);

            actionRequest.QuestionOptionsDtos?.ForEach(x => x.QuestionId = id);

            actionRequest.QuestionSourcewareDtos?.ForEach(x => x.QuestionId = id);

            actionRequest.QuestionTagDtos?.ForEach(x => x.QuestionId = id);

            if (actionRequest.QuestionAnswerDto != null)
            {
                actionRequest.QuestionAnswerDto.QuestionId = id;
            }
        }

        private void ResolveQuestionId(UpdateQuestionDto actionRequest, string id)
        {
            actionRequest.QuestionKnowladgeDtos?.ForEach(x => x.QuestionId = id);

            actionRequest.QuestionOptionsDtos?.ForEach(x => x.QuestionId = id);

            actionRequest.QuestionSourcewareDtos?.ForEach(x => x.QuestionId = id);

            actionRequest.QuestionTagDtos?.ForEach(x => x.QuestionId = id);

            if (actionRequest.QuestionAnswerDto != null)
            {
                actionRequest.QuestionAnswerDto.QuestionId = id;
            }
        }

        private async Task<List<QuestionTag>> AddQuestionTags(AddQuestionDto actionRequest, CancellationToken cancellationToken)
        {
            if (actionRequest.QuestionTagDtos == null) return null;

            actionRequest.QuestionTagDtos.ResolveOperationStatus();

            var questionTagDtos = actionRequest.QuestionTagDtos.Where(x => x.OperationStatus == EEOperationStatus.Add).ToList();

            var questionTags = _mapper.Map<List<QuestionTag>>(questionTagDtos);

            questionTags.ToInsert();

            await _questionTagRepository.ValidateAddAsync(questionTags, cancellationToken);

            return questionTags;
        }

        private async Task<List<QuestionOptions>> AddQuestionOptions(AddQuestionDto actionRequest, CancellationToken cancellationToken)
        {
            if (actionRequest.QuestionOptionsDtos == null) return null;

            actionRequest.QuestionOptionsDtos.ResolveOperationStatus();
            // 简答和填空没有选项
            if (actionRequest.QuestionType == Share.Enums.Exams.EQuestionType.Essay || actionRequest.QuestionType == Share.Enums.Exams.EQuestionType.Blank)
                return null;

            var questionOptionseDtos = actionRequest.QuestionOptionsDtos.Where(x => x.OperationStatus == EEOperationStatus.Add).ToList();

            
            var questionOptionses = _mapper.Map<List<QuestionOptions>>(questionOptionseDtos);

            var sortIndex = 0;
            questionOptionses.ForEach(m =>
            {
                m.SortIndex = sortIndex;
                m.Label = SystemConstants.Labals[sortIndex];
                sortIndex++;
            });

            questionOptionses.ToInsert();

            await _questionOptionRepository.ValidateAddAsync(questionOptionses, cancellationToken);

            return questionOptionses;
        }
        private async Task<QuestionAnswer> AddQuestionAnswer(AddQuestionDto actionRequest, CancellationToken cancellationToken)
        {
            if (actionRequest.QuestionAnswerDto == null) return null;

            if (actionRequest.QuestionAnswerDto.Answer != null)
            {
                actionRequest.QuestionAnswerDto.OperationStatus = EEOperationStatus.Add;
            }

            // 简答和填空没有选项
            if (actionRequest.QuestionType.CheckSelectType()) return null;
            var questionAnswer = _mapper.Map<QuestionAnswer>(actionRequest.QuestionAnswerDto);

            questionAnswer.ToInsert();

            await _questionAnswerRepository.ValidateAddAsync(questionAnswer, cancellationToken);

            return questionAnswer;
        }
        private async Task<List<QuestionSourceware>> AddSourcewares(AddQuestionDto actionRequest, CancellationToken cancellationToken)
        {
            if (actionRequest.QuestionSourcewareDtos == null) return null;

            actionRequest.QuestionSourcewareDtos.ResolveOperationStatus();

            var questionSourcewareDtos = actionRequest.QuestionSourcewareDtos.Where(x => x.OperationStatus == EEOperationStatus.Add).ToList();

            var questionSourcewares = _mapper.Map<List<QuestionSourceware>>(questionSourcewareDtos);
                        
            questionSourcewares.ToInsert();

            await _questionSourcewareRepository.ValidateAddAsync(questionSourcewares, cancellationToken);

            return questionSourcewares;
        }
        private async Task<List<QuestionKnowladge>> AddKnowladges(AddQuestionDto actionRequest, CancellationToken cancellationToken)
        {
            if (actionRequest.QuestionKnowladgeDtos == null) return null;

            actionRequest.QuestionKnowladgeDtos.ResolveOperationStatus();

            var questionKnoladgeDtos = actionRequest.QuestionKnowladgeDtos.Where(x => x.OperationStatus == EEOperationStatus.Add).ToList();

            var questionKnowladges = _mapper.Map<List<QuestionKnowladge>>(questionKnoladgeDtos);

            questionKnowladges.ToInsert();

            await _questionKnowladgeRepository.ValidateAddAsync(questionKnowladges, cancellationToken);

            return questionKnowladges;
        }
        private async Task<List<QuestionSourceware>> UpdateSourcewares(UpdateQuestionDto actionRequest, List<QuestionSourceware> all, CancellationToken cancellationToken)
        {
            if (actionRequest.QuestionSourcewareDtos == null) return null;

            actionRequest.QuestionSourcewareDtos.ResolveOperationStatus();

            var questionSourcewareDtos = actionRequest.QuestionSourcewareDtos.Where(x => x.OperationStatus == EEOperationStatus.Update).ToList();

            var ids = questionSourcewareDtos.Select(x => x.Id);

            var questionSourcewares = all.Where(x => ids.Contains(x.Id)).ToList();

            var entitys = new List<QuestionSourceware>();
            foreach (var questionSourcewareDto in questionSourcewareDtos)
            {
                var entity = questionSourcewares.FirstOrDefault(x => x.Id == questionSourcewareDto.Id);
                if (entity != null)
                {
                    entity.QuestionId = actionRequest.Id;
                    entitys.Add(_mapper.Map<UpdateQuestionSourcewareDto, QuestionSourceware>(questionSourcewareDto, entity));
                }

            }

            //questionSourcewares = _mapper.Map<List<UpdateQuestionSourcewareDto>,List<QuestionSourceware>>(questionSourcewareDtos,questionSourcewares);

            //questionSourcewares.ForEach(x => x.QuestionId = actionRequest.Id);

            questionSourcewares.ToUpdate();

            await _questionSourcewareRepository.ValidateUpdateAsync(questionSourcewares, cancellationToken);

            return questionSourcewares;
        }

        private async Task<List<QuestionKnowladge>> UpdateKnowladges(UpdateQuestionDto actionRequest, List<QuestionKnowladge> all, CancellationToken cancellationToken)
        {
            if (actionRequest.QuestionKnowladgeDtos == null) return null;

            actionRequest.QuestionKnowladgeDtos.ResolveOperationStatus();

            var questionKnowladgeDtos = actionRequest.QuestionKnowladgeDtos.Where(x => x.OperationStatus == EEOperationStatus.Add).ToList();

            var ids = questionKnowladgeDtos.Select(x => x.Id);

            var questionKnowladges = all.Where(x => ids.Contains(x.Id)).ToList();

            var entitys = new List<QuestionKnowladge>();
            foreach (var questionKnowladgeDto in questionKnowladgeDtos)
            {
                var entity = questionKnowladges.FirstOrDefault(x => x.Id == questionKnowladgeDto.Id);
                if (entity != null)
                {
                    entity.QuestionId = actionRequest.Id;
                    entitys.Add(_mapper.Map<UpdateQuestionKnowladgeDto, QuestionKnowladge>(questionKnowladgeDto, entity));
                }
               
            }


            //questionKnowladges = _mapper.Map<List<UpdateQuestionKnowladgeDto>, List<QuestionKnowladge>>(questionKnowladgeDtos,questionKnowladges);

            //questionKnowladges.ForEach(x => x.QuestionId = actionRequest.Id);

            questionKnowladges.ToUpdate();

            await _questionKnowladgeRepository.ValidateUpdateAsync(questionKnowladges, cancellationToken);

            return questionKnowladges;
        }
        private async Task<QuestionAnswer> UpdateQuestionAnswer(UpdateQuestionDto actionRequest, CancellationToken cancellationToken)
        {
            if (actionRequest.QuestionAnswerDto == null) return null;

            // 简单和填空没有选项
            if (actionRequest.QuestionType.CheckSelectType()) return null;

            var questionAnswer = _mapper.Map<QuestionAnswer>(actionRequest.QuestionAnswerDto);

            questionAnswer.QuestionId = actionRequest.Id;            

            questionAnswer.ToUpdate();

            await _questionAnswerRepository.ValidateUpdateAsync(questionAnswer, cancellationToken);

            return questionAnswer;
        }
        private async Task<List<QuestionOptions>> UpdateQuestionOptions(UpdateQuestionDto actionRequest,List<QuestionOptions> all, CancellationToken cancellationToken)
        {
            if (actionRequest.QuestionOptionsDtos == null) return null;

            // 简单和填空没有选项
            if (actionRequest.QuestionType == Share.Enums.Exams.EQuestionType.Essay || actionRequest.QuestionType == Share.Enums.Exams.EQuestionType.Blank)
                return null;

            var questionOptionsDtos = actionRequest.QuestionOptionsDtos.Where(x => x.OperationStatus == EEOperationStatus.Update).ToList();

            var ids = questionOptionsDtos.Select(x => x.Id);

            var questionOptionses = all.Where(x => ids.Contains(x.Id)).ToList();

            var entitys = new List<QuestionOptions>();
            foreach(var questionOptionsDto in questionOptionsDtos)
            {
                var entity = questionOptionses.FirstOrDefault(x => x.Id == questionOptionsDto.Id);
                if (entity != null)
                {
                    entity.QuestionId = actionRequest.Id;
                    entitys.Add(_mapper.Map<UpdateQuestionOptionsDto, QuestionOptions>(questionOptionsDto, entity));
                }               
            }

            //questionOptionses =  _mapper.Map<List<QuestionOptionsDto>, List<QuestionOptions>>(questionOptionsDtos,questionOptionses);

            //entitys.ForEach(x => x.QuestionId = actionRequest.Id);

            entitys.ToUpdate();

            await _questionOptionRepository.ValidateUpdateAsync(entitys, cancellationToken);

            return entitys;
        }

        private async Task<List<QuestionTag>> UpdateQuestionTags(UpdateQuestionDto actionRequest,List<QuestionTag> all, CancellationToken cancellationToken)
        {
            if (actionRequest.QuestionTagDtos == null) return null;

            var questionTagDtos = actionRequest.QuestionTagDtos.Where(x => x.OperationStatus == EEOperationStatus.Update).ToList();

            var ids = questionTagDtos.Select(x => x.Id);

            var questionTags = all.Where(x => ids.Contains(x.Id)).ToList();

            var entitys = new List<QuestionTag>();
            foreach (var questionOptionsDto in questionTagDtos)
            {
                var entity = questionTags.FirstOrDefault(x => x.Id == questionOptionsDto.Id);
                if (entity != null)
                {
                    entity.QuestionId = actionRequest.Id;
                    entitys.Add(_mapper.Map<UpdateQuestionTagDto, QuestionTag>(questionOptionsDto, entity));
                }
                
            }
            //questionTags = _mapper.Map<List<UpdateQuestionTagDto>, List<QuestionTag>>(questionTagDtos,questionTags);

            questionTags.ToUpdate();

            await _questionTagRepository.ValidateUpdateAsync(questionTags, cancellationToken);

            return questionTags;
        }
        private async Task DeleteSourcewares(EntityQueryRequest entityQueryRequest, CancellationToken cancellationToken)
        {
            await _questionSourcewareRepository.DeleteWithValidateAsync(entityQueryRequest, cancellationToken);
        }
        private async Task DeleteKnowladges(EntityQueryRequest entityQueryRequest, CancellationToken cancellationToken)
        {
            await _questionKnowladgeRepository.DeleteWithValidateAsync(entityQueryRequest, cancellationToken);
        }
        private async Task DeleteQuestionAnswer(EntityQueryRequest entityQueryRequest, CancellationToken cancellationToken)
        {
            await _questionAnswerRepository.DeleteWithValidateAsync(entityQueryRequest, cancellationToken);
        }
        private async Task DeleteQuestionOptions(EntityQueryRequest entityQueryRequest, CancellationToken cancellationToken)
        {
            await _questionOptionRepository.DeleteWithValidateAsync(entityQueryRequest, cancellationToken);
        }
        private async Task DeleteQuestionTags(EntityQueryRequest entityQueryRequest, CancellationToken cancellationToken)
        {
            await _questionTagRepository.DeleteWithValidateAsync(entityQueryRequest, cancellationToken);
        }
        private async Task<List<QuestionSourceware>> ModifySourcewares(UpdateQuestionDto actionRequest, CancellationToken cancellationToken)
        {
            var all = await _questionSourcewareRepository.Queryable().Where(x => x.QuestionId == actionRequest.Id).ToListAsync();

            if (actionRequest.QuestionSourcewareDtos == null) return null;

            var questionSourcewares = new List<QuestionSourceware>();

            actionRequest.QuestionSourcewareDtos.ResolveOperationStatus(all);

            questionSourcewares.AddRangeExt(await AddSourcewares(actionRequest, cancellationToken));

            questionSourcewares.AddRangeExt(await UpdateSourcewares(actionRequest, all, cancellationToken));

            var questionSourcewareDtos = actionRequest.QuestionSourcewareDtos.Where(x => x.OperationStatus == EEOperationStatus.Delete);
            var ids = questionSourcewareDtos.Select(m => m.Id);
            EntityQueryRequest entityQueryRequest = ResovleDelete<QuestionSourceware>(ids);

            await DeleteSourcewares(entityQueryRequest, cancellationToken);

            return questionSourcewares;
        }

        private async Task<List<QuestionKnowladge>> ModifyKnowladges(UpdateQuestionDto actionRequest, CancellationToken cancellationToken)
        {
            var all = await _questionKnowladgeRepository.Queryable().Where(x => x.QuestionId == actionRequest.Id).ToListAsync();

            if (actionRequest.QuestionKnowladgeDtos == null) return null;

            var questionKnowladges = new List<QuestionKnowladge>();

            actionRequest.QuestionKnowladgeDtos.ResolveOperationStatus(all);

            questionKnowladges.AddRangeExt(await AddKnowladges(actionRequest, cancellationToken));

            questionKnowladges.AddRangeExt(await UpdateKnowladges(actionRequest, all, cancellationToken));

            var questionKnowladgeDtos = actionRequest.QuestionKnowladgeDtos.Where(x => x.OperationStatus == EEOperationStatus.Delete);
            var ids = questionKnowladgeDtos.Select(m => m.Id);
            EntityQueryRequest entityQueryRequest = ResovleDelete<QuestionKnowladge>(ids);

            await DeleteKnowladges(entityQueryRequest, cancellationToken);

            return questionKnowladges;
        }

        private async Task<QuestionAnswer> ModifyQuestionAnswer(UpdateQuestionDto actionRequest, CancellationToken cancellationToken)
        {
            var all = await _questionAnswerRepository.Queryable().Where(x => x.QuestionId == actionRequest.Id).ToListAsync();

            var questionAnswer = new QuestionAnswer();

            if (actionRequest.QuestionAnswerDto == null)
            {
                if (all == null) 
                    return null;
                else
                {
                    var ids = all.Select(m => m.Id).ToList();
                    var entityQueryRequest = new EntityQueryRequest
                    {
                        Expression = ExpressionableUtility.CreateExpression<QuestionAnswer>()
                        .AndIF(ids.IsNotEmpty(), x => ids.Contains(x.Id)).ToExpression()
                    };

                    await DeleteQuestionAnswer(entityQueryRequest, cancellationToken);

                    return null;
                }

            }

            if (actionRequest.QuestionAnswerDto.Id != null)
            {
                actionRequest.QuestionAnswerDto.OperationStatus = EEOperationStatus.Update;
            }

            var add = await AddQuestionAnswer(actionRequest, cancellationToken);

            if (add!=null)
            {
                questionAnswer = add;
            }

            var update= await UpdateQuestionAnswer(actionRequest, cancellationToken);

            if(update != null)
            {
                questionAnswer = update;
            }

            if (actionRequest.QuestionAnswerDto != null && actionRequest.QuestionAnswerDto.OperationStatus == EEOperationStatus.Delete)
            {
                var entityQueryRequest = new EntityQueryRequest
                {
                    Id = actionRequest.QuestionAnswerDto?.Id
                };

                await DeleteQuestionAnswer(entityQueryRequest, cancellationToken);
            }

            return questionAnswer;

        }

        private async Task<List<QuestionOptions>> ModifyQuestionOptions(UpdateQuestionDto actionRequest, CancellationToken cancellationToken)
        {
            if (actionRequest.QuestionOptionsDtos == null) return null;

            var questionOptions = new List<QuestionOptions>();

            var all = await _questionOptionRepository.Queryable().Where(x=>x.QuestionId == actionRequest.Id).ToListAsync();

            actionRequest.QuestionOptionsDtos.ResolveOperationStatus(all);

            questionOptions.AddRangeExt(await AddQuestionOptions(actionRequest, cancellationToken));

            questionOptions.AddRangeExt(await UpdateQuestionOptions(actionRequest, all, cancellationToken));

            var questionOptionsDtos = actionRequest.QuestionOptionsDtos.Where(x => x.OperationStatus == EEOperationStatus.Delete);
            var ids = questionOptionsDtos.Select(m => m.Id);
            EntityQueryRequest entityQueryRequest = ResovleDelete<QuestionOptions>(ids);

            await DeleteQuestionOptions(entityQueryRequest, cancellationToken);

            return questionOptions;
        }

        private async Task<List<QuestionTag>> ModifyQuestionTags(UpdateQuestionDto actionRequest, CancellationToken cancellationToken)
        {
            if (actionRequest.QuestionTagDtos == null) return null;

            var questionTags = new List<QuestionTag>();

            var all = await _questionTagRepository.Queryable().Where(x => x.QuestionId == actionRequest.Id).ToListAsync();

            actionRequest.QuestionTagDtos.ResolveOperationStatus(all);

            questionTags.AddRangeExt(await AddQuestionTags(actionRequest, cancellationToken));

            questionTags.AddRangeExt(await UpdateQuestionTags(actionRequest, all, cancellationToken));

            var questionTagDtos = actionRequest.QuestionTagDtos.Where(x => x.OperationStatus == EEOperationStatus.Delete);
            var ids = questionTagDtos.Select(m => m.Id);
            EntityQueryRequest entityQueryRequest = ResovleDelete<QuestionTag>(ids);

            await DeleteQuestionTags(entityQueryRequest, cancellationToken);

            return questionTags;
        }

        private EntityQueryRequest ResovleDelete<T>(IEnumerable<string> ids) where T:class,IEntity<string>,new()
        {
            Expressionable<T> expressionable = ExpressionableUtility.CreateExpression<T>();
            expressionable.AndIF(ids.Any(), x => ids.Contains(x.Id));

            var entityQueryRequest = new EntityQueryRequest
            {
                Expression = ids.Any() ? expressionable.ToExpression() : null
            };
            return entityQueryRequest;
        }

        private async Task<List<QuestionSourcewareDto>> GetQuestionSourcewares(EntityQueryRequest entityQueryRequest)
        {
            var questionSourcewareTable = _questionSourcewareRepository.Queryable().Where(x => x.QuestionId == entityQueryRequest.Id);

            var sourcewareTable = new ExamRepository<Sourceware>(_questionSourcewareRepository.UOW, _dataPermissionFilterBuilder, _serviceProvider).Queryable();

            var questionSourcewares = questionSourcewareTable.InnerJoin(sourcewareTable, (q, s) =>
              q.SourcewareId == s.Id).Select((q, s) => new QuestionSourcewareDto
              {
                  Id = q.Id,
                  SourcewareId = q.SourcewareId,
                  Name = s.Name
              });

            return await questionSourcewares.ToListAsync();
        }

        private async Task<List<QuestionOptionsDto>> GetQuestionOptions(EntityQueryRequest entityQueryRequest)
        {
            var questionOptionTable = await _questionOptionRepository.Queryable().Where(x => x.QuestionId == entityQueryRequest.Id).ToListAsync();

            var questionOptions = _mapper.Map<List<QuestionOptionsDto>>(questionOptionTable);

            return questionOptions;
        }

        private async Task<List<QuestionKnowladgeDto>> GetKnowladges(EntityQueryRequest entityQueryRequest)
        {
            var questionKnowladgeTable = await _questionKnowladgeRepository.Queryable().Where(x => x.QuestionId == entityQueryRequest.Id).ToListAsync();

            var questionKnowladges = _mapper.Map<List<QuestionKnowladgeDto>>(questionKnowladgeTable);

            return questionKnowladges;
        }

        private async Task<QuestionAnswerDto> GetQuestionAnswer(EntityQueryRequest entityQueryRequest)
        {
            var questionAnswer = await _questionAnswerRepository.GetAsync(entityQueryRequest.Id);

            var questionAnswerDto = _mapper.Map<QuestionAnswerDto>(questionAnswer);

            return questionAnswerDto;
        }

        private async Task<List<QuestionTagDto>> GetQuestionTags(EntityQueryRequest entityQueryRequest)
        {
            var questionTags = _questionTagRepository.Queryable().Where(x => x.QuestionId == entityQueryRequest.Id);

            var examTags = new ExamRepository<ExamTag>(_questionTagRepository.UOW, _dataPermissionFilterBuilder, _serviceProvider).Queryable();

            var questionTagDtos = questionTags.InnerJoin(examTags, (q, t) => q.TagId == t.Id).Select((q, t) => new QuestionTagDto
            {
                Id = q.Id,
                TagId = q.TagId,
                Tag = t.Name,
                QuestionId = q.QuestionId
            });

            return await questionTagDtos.ToListAsync();
        }
        #endregion

        #region protected method

        //protected override async Task CompleteAdd(Question entity)
        //{
        //    if (entity.QuestionType.CheckSelectType())
        //    {
        //        await base.AddNav(entity)
        //        .Include(x => x.QuestionTags)
        //        .Include(x => x.QuestionOptionses)
        //        .Include(x => x.QuestionSourcewares)
        //        .Include(x => x.QuestionKnowladges).ExecuteCommandAsync();
        //    }
        //    else
        //    {
        //        await base.AddNav(entity)
        //       .Include(x => x.QuestionTags)
        //       .Include(x => x.QuestionAnswerE)
        //       .Include(x => x.QuestionSourcewares)
        //       .Include(x => x.QuestionKnowladges).ExecuteCommandAsync();
        //    }
        //}

        protected override async Task CompleteUpdate(Question entity)
        {
            if (entity.QuestionType.CheckSelectType())
            {
                await base.UpdateNav(entity)
                .Include(x => x.QuestionTags,new UpdateNavOptions
                {
                    OneToManyInsertOrUpdate = true
                })
                .Include(x => x.QuestionOptionses, new UpdateNavOptions
                {
                    OneToManyInsertOrUpdate = true
                })
                .Include(x => x.QuestionSourcewares, new UpdateNavOptions
                {
                    OneToManyInsertOrUpdate = true
                })
                .Include(x => x.QuestionKnowladges, new UpdateNavOptions
                {
                    OneToManyInsertOrUpdate = true
                }).ExecuteCommandAsync();
            }
            else
            {
                await base.UpdateNav(entity)
               .Include(x => x.QuestionTags, new UpdateNavOptions
               {
                   OneToManyInsertOrUpdate = true
               })
               .Include(x => x.QuestionAnswerE)
               .Include(x => x.QuestionSourcewares, new UpdateNavOptions
               {
                   OneToManyInsertOrUpdate = true
               })
               .Include(x => x.QuestionKnowladges, new UpdateNavOptions
               {
                   OneToManyInsertOrUpdate = true
               }).ExecuteCommandAsync();
            }
        }

        #endregion
    }
}
