﻿using Exam.Application.Interface.Train;
using Exam.Infrastructure.Data.Entity;
using Exam.Infrastructure.Enums;
using Exam.Infrastructure.Extensions;
using Exam.Infrastructure.Web.Extensions;
using Exam.Infrastructure.Web.Utilities;
using Exam.Insfrastructure.Service.Service;
using Exam.Repository.Sqlsugar.Repositories;
using Exam.Share;
using Exam.Share.ViewResponses.Train;
using Exam.Trains;
using Hotline.Application.Exam.QueryExtensions.Trains;
using Hotline.Exams.Trains;
using Hotline.Repository.SqlSugar;
using Hotline.Repository.SqlSugar.DataPermissions;
using Hotline.Repository.SqlSugar.Exam.Interfaces.Trains;
using Hotline.Repository.SqlSugar.Extensions;
using Hotline.Share.Dtos.Trains;
using Hotline.Share.Requests.Train;
using JiebaNet.Segmenter.Common;
using MapsterMapper;
using XF.Domain.Dependency;

namespace Hotline.Application.Exam.Service.Trains
{
    public class TrainPlanService : ApiService<TrainPlan, AddTrainPlanDto, UpdateTrainPlanDto, HotlineDbContext>, ITrainPlanService, IScopeDependency
    {
        private readonly ITrainPlanRepository _repository;
        private readonly ITrainPlanTemplateRepository _trainPlanTemplateRepository;
        private readonly ITrainRecordRepository _trainRecordRepository;
        private readonly IMapper _mapper;

        public TrainPlanService(ITrainPlanRepository repository,
            ITrainPlanTemplateRepository trainPlanTemplateRepository,
            ITrainRecordRepository trainRecordRepository,
            IMapper mapper) : base(repository, mapper)
        {
            this._repository = repository;
            this._trainPlanTemplateRepository = trainPlanTemplateRepository;
            this._trainRecordRepository = trainRecordRepository;
            this._mapper = mapper;
        }

        #region public mehtod
        public async Task<TrainPlanDto> GetAsync(EntityQueryRequest entityQueryRequest)
        {

            var trainPlan = await _repository.GetAsync(entityQueryRequest.Id);

            var trainPlanDto = _mapper.Map<TrainPlanDto>(trainPlan);

            trainPlanDto.TrainPlanTemplateDtos = await GetTrainPlanTemplates(entityQueryRequest);

            trainPlanDto.TrainUserDtos = await GetTrainRecords(entityQueryRequest);

            return trainPlanDto;
        }

        public async Task<(int, List<TrainPlanViewResponse>)> GetListAsync(TrainPlanPagedRequest queryRequest)
        {
            SqlSugar.ISugarQueryable<TrainPlanViewResponse> queryResult = QueryRequest(queryRequest);

            var total = await queryResult.CountAsync();
            var items = await queryResult.ToListAsync();

            return (total, items);
        }

        public async Task<PageViewResponse<TrainPlanViewResponse>> GetPagedListAsync(TrainPlanPagedRequest queryRequest)
        {
            SqlSugar.ISugarQueryable<TrainPlanViewResponse> queryResult = QueryRequest(queryRequest);

            var total = await queryResult.CountAsync();
            var items = await queryResult.ToPageListAsync(queryRequest.PageIndex, queryRequest.PageSize);

            return new TrainPlanPageViewResponse
            {
                Items = items,
                Pagination = new Pagination(queryRequest.PageIndex, queryRequest.PageSize, total)
            };

        }

        public override async Task<string> AddAsync(AddTrainPlanDto actionRequest, CancellationToken cancellationToken)
        {
            var id = await base.AddAsync(actionRequest, cancellationToken);

            ResolveTemplateId(actionRequest, id);

            await AddTrainPlanTemplatesAsync(actionRequest, cancellationToken);

            await AddTrainRecordsAsync(actionRequest, cancellationToken);

            return id;

        }

        public override async Task UpdateAsync(UpdateTrainPlanDto actionRequest, CancellationToken cancellationToken)
        {
            await base.UpdateAsync(actionRequest, cancellationToken);

            await ModifyTrainPlanTemplatesAsync(actionRequest, cancellationToken);

            await ModifyTrainRecordsAsync(actionRequest, cancellationToken);
        }



        public override async Task DeleteAsync(EntityQueryRequest entityQueryRequest, CancellationToken cancellationToken)
        {
            await base.DeleteAsync(entityQueryRequest, cancellationToken);

            var tmpEntityQueryRequest = ExpressionableUtility.CreateExpression<TrainPlanTemplate>()
               .AndIF(entityQueryRequest.Id.IsNotNullOrEmpty(), x => x.TrainPlanId == entityQueryRequest.Id)
               .AndIF(entityQueryRequest.Ids.IsNotNull(), x => entityQueryRequest.Ids.Contains(x.TrainPlanId))
               .ToEntityQueryRequest<TrainPlanTemplate>();

            await DeleteTrainPlanTemplateAsync(tmpEntityQueryRequest, cancellationToken);

            tmpEntityQueryRequest = ExpressionableUtility.CreateExpression<TrainRecord>()
               .AndIF(entityQueryRequest.Id.IsNotNullOrEmpty(), x => x.TrainPlanId == entityQueryRequest.Id)
               .AndIF(entityQueryRequest.Ids.IsNotNull(), x => entityQueryRequest.Ids.Contains(x.TrainPlanId))
               .ToEntityQueryRequest<TrainRecord>();

            await DeleteTrainRecordsAsync(tmpEntityQueryRequest, cancellationToken);

        }
        #endregion

        #region
        private async Task<List<TrainPlanTemplateDto>> GetTrainPlanTemplates(EntityQueryRequest entityQueryRequest)
        {
            var trainPlanTemplateTable = _trainPlanTemplateRepository.Queryable().Where(x => x.TrainPlanId == entityQueryRequest.Id);

            var queryResult = trainPlanTemplateTable.Select(t => new TrainPlanTemplateDto
            {
                TrainPlanId = t.TrainPlanId,
                TrainTemplateId = t.TraimTemplateId,
            }); 

            return await queryResult.ToListAsync();

        }

        private void ResolveTemplateId(AddTrainPlanDto actionRequest, string id)
        {
            if (actionRequest.TrainPlanTemplateDtos != null)
            {
                actionRequest.TrainPlanTemplateDtos.ForEach(x => x.TrainPlanId = id);
            }

            if (actionRequest.TrainUserDtos != null)
            {
                actionRequest.TrainUserDtos.ForEach(x => x.TrainPlanId = id);
            }
        }

        private async Task<List<TrainRecordDto>> GetTrainRecords(EntityQueryRequest entityQueryRequest)
        {
           
            var trainPracticeTable = _trainRecordRepository.Queryable().Where(x => x.TrainPlanId == entityQueryRequest.Id);

            var trainPractices = trainPracticeTable.Select(t => new TrainRecordDto
            {
                Id = t.Id,
                TrainPlanId = t.TrainPlanId,
                UserId = t.UserId
            });

            return await trainPractices.ToListAsync();
        }

        private SqlSugar.ISugarQueryable<TrainPlanViewResponse> QueryRequest(TrainPlanPagedRequest queryRequest)
        {
            var expression = queryRequest.GetExpression();
            var queryable = _repository.Queryable().Where(expression);

            var trainPlanTemplateExpression = queryRequest.GetTemplateExpression();
            var trainPlanTemplateTable = _trainPlanTemplateRepository.Queryable().Where(trainPlanTemplateExpression);
            

            var queryResult = queryable.InnerJoin(trainPlanTemplateTable,(q,t)=>q.Id == t.TrainPlanId).Select((q,t) => new TrainPlanViewResponse
            {
                Id = q.Id,
                Code = q.Code,
                Name = q.Name,
                TrainStartTime = q.TrainStartTime,
                TrainEndTime = q.TrainEndTime,
                SortIndex = q.SortIndex,
                Status = q.Status,
                CreationTime = q.CreationTime,
                CreatorName = q.CreatorName,
                CreatorOrgName = q.CreatorOrgName
            });
            return queryResult;
        }

        private async Task AddTrainPlanTemplatesAsync(AddTrainPlanDto actionRequest, CancellationToken cancellationToken)
        {
            if (actionRequest.TrainPlanTemplateDtos != null)
            {
                var trainPlanTemplateDtos = actionRequest.TrainPlanTemplateDtos.Where(x => x.OperationStatus == EEOperationStatus.Add).ToList();

                var trainPlanTemplates = _mapper.Map<List<TrainPlanTemplate>>(trainPlanTemplateDtos);

                trainPlanTemplates.ToInsert();

                await _trainPlanTemplateRepository.AddWithValidateAsync(trainPlanTemplates, cancellationToken);
            }
        }

        private async Task AddTrainRecordsAsync(AddTrainPlanDto actionRequest, CancellationToken cancellationToken)
        {
            if (actionRequest.TrainUserDtos != null)
            {
                var trainPracticeDtos = actionRequest.TrainUserDtos.Where(x => x.OperationStatus == EEOperationStatus.Add).ToList();

                var trainPractices = _mapper.Map<List<TrainRecord>>(trainPracticeDtos);

                trainPractices.ToInsert();

                await _trainRecordRepository.AddWithValidateAsync(trainPractices, cancellationToken);
            }
        }

        private async Task ModifyTrainPlanTemplatesAsync(UpdateTrainPlanDto actionRequest, CancellationToken cancellationToken)
        {
            await AddTrainPlanTemplatesAsync(actionRequest, cancellationToken);

            await UpdateTrainPlanTemplatesAsync(actionRequest, cancellationToken);

            if (actionRequest.TrainUserDtos != null)
            {
                var ids = actionRequest.TrainUserDtos.Where(x => x.OperationStatus == EEOperationStatus.Delete).Select(x => x.Id).ToList();

                var entityQueryRequest = new EntityQueryRequest
                {
                    Expression = ExpressionableUtility.CreateExpression<TrainPlanTemplate>()
                    .AndIF(ids.IsNotEmpty(), x => ids.Contains(x.Id))
                    .ToExpression()
                };

                await DeleteTrainPlanTemplateAsync(entityQueryRequest, cancellationToken);
            }


        }

        private async Task DeleteTrainPlanTemplateAsync(EntityQueryRequest entityQueryRequest, CancellationToken cancellationToken)
        {
            await _trainPlanTemplateRepository.DeleteWithValidateAsync(entityQueryRequest, cancellationToken);
        }

        private async Task UpdateTrainPlanTemplatesAsync(UpdateTrainPlanDto actionRequest, CancellationToken cancellationToken)
        {
            if (actionRequest.TrainUserDtos == null) return;

            var trainPracticeDtos = actionRequest.TrainUserDtos.Where(x => x.OperationStatus == EEOperationStatus.Update).ToList();

            var ids = trainPracticeDtos.Select(x => x.Id);

            var trainUsers = await _trainRecordRepository.Queryable().Where(x => ids.Contains(x.Id)).ToListAsync();

            trainUsers = _mapper.Map<List<UpdateTrainRecordDto>, List<TrainRecord>>(trainPracticeDtos, trainUsers);

            trainUsers.ToUpdate();

            await _trainRecordRepository.UpdateWithValidateAsync(trainUsers, cancellationToken);
        }

        private async Task ModifyTrainRecordsAsync(UpdateTrainPlanDto actionRequest, CancellationToken cancellationToken)
        {
            await AddTrainRecordsAsync(actionRequest, cancellationToken);

            await UpdateTrainRecordsAsync(actionRequest, cancellationToken);

            if (actionRequest.TrainUserDtos != null)
            {
                var ids = actionRequest.TrainUserDtos.Where(x => x.OperationStatus == EEOperationStatus.Delete).Select(x => x.Id).ToList();

                var entityQueryRequest = new EntityQueryRequest
                {
                    Expression = ExpressionableUtility.CreateExpression<TrainRecord>()
                    .AndIF(ids.IsNotEmpty(), x => ids.Contains(x.Id))
                    .ToExpression()
                };

                await DeleteTrainRecordsAsync(entityQueryRequest, cancellationToken);
            }
        }

        private async Task DeleteTrainRecordsAsync(EntityQueryRequest entityQueryRequest, CancellationToken cancellationToken)
        {
            await _trainPlanTemplateRepository.DeleteWithValidateAsync(entityQueryRequest, cancellationToken);
        }

        private async Task UpdateTrainRecordsAsync(UpdateTrainPlanDto actionRequest, CancellationToken cancellationToken)
        {
            if (actionRequest.TrainPlanTemplateDtos == null) return;

            var trainPlanTemplateDtos = actionRequest.TrainPlanTemplateDtos.Where(x => x.OperationStatus == EEOperationStatus.Update).ToList();

            var ids = trainPlanTemplateDtos.Select(x => x.Id);

            var trainPlanTemplates = await _trainPlanTemplateRepository.Queryable().Where(x => ids.Contains(x.Id)).ToListAsync();

            trainPlanTemplates = _mapper.Map<List<UpdateTrainPlanTemplateDto>, List<TrainPlanTemplate>>(trainPlanTemplateDtos, trainPlanTemplates);

            trainPlanTemplates.ToUpdate();

            await _trainPlanTemplateRepository.UpdateWithValidateAsync(trainPlanTemplates, cancellationToken);
        }
        #endregion
    }
}
