﻿using DocumentFormat.OpenXml.Office2010.Excel;
using Exam.Application.Interface.Train;
using Exam.Infrastructure.Data.Entity;
using Exam.Infrastructure.Enums;
using Exam.Infrastructure.Extensions;
using Exam.Infrastructure.Web.Extensions;
using Exam.Infrastructure.Web.Utilities;
using Exam.Insfrastructure.Service.Service;
using Exam.Practices;
using Exam.Questions;
using Exam.Repository.Sqlsugar.Repositories;
using Exam.Share.ViewResponses.Train;
using Exam.Trains;
using Hotline.Application.Exam.Core.Extensions;
using Hotline.Application.Exam.QueryExtensions.Trains;
using Hotline.Exams.TestPapers;
using Hotline.Exams.Trains;
using Hotline.Repository.SqlSugar;
using Hotline.Repository.SqlSugar.DataPermissions;
using Hotline.Repository.SqlSugar.Exam.Interfaces.Questions;
using Hotline.Repository.SqlSugar.Exam.Interfaces.Trains;
using Hotline.Repository.SqlSugar.Extensions;
using Hotline.Repository.SqlSugar.Interface;
using Hotline.Share.Dtos.Questions;
using Hotline.Share.Dtos.Trains;
using Hotline.Share.Requests.Train;
using JiebaNet.Segmenter.Common;
using MapsterMapper;
using XF.Domain.Dependency;

namespace Hotline.Application.Exam.Service.Trains
{
    public class TrainTemplateService : ApiService<TrainTemplate, AddTrainTemplateDto,UpdateTrainTemplateDto, HotlineDbContext>, ITrainTemplateService, IScopeDependency
    {
        private readonly ITrainTemplateRepository _repository;
        private readonly ITrainKnowladgeRepository _trainKnowladgeRepository;
        private readonly ITrainPracticeRepository _trainPracticeRepository;
        private readonly ITrainPracticeOptionsRepository _trainPracticeOptionsRepository;
        private readonly ITrainPracticeSourcewareRepository _trainPracticeSourcewareRepository;
        private readonly ITrainPracticeKnowladgeRepository _trainPracticeKnowladgeRepository;
        private readonly IDataPermissionFilterBuilder _dataPermissionFilterBuilder;
        private readonly IServiceProvider _serviceProvider;
        private readonly IMapper _mapper
            ;

        public TrainTemplateService(ITrainTemplateRepository repository, 
            ITrainKnowladgeRepository trainKnowladgeRepository,
            ITrainPracticeRepository trainPracticeRepository,
            ITrainPracticeOptionsRepository trainPracticeOptionsRepository,
            ITrainPracticeSourcewareRepository trainPracticeSourcewareRepository,
            ITrainPracticeKnowladgeRepository trainPracticeKnowladgeRepository,
            IDataPermissionFilterBuilder dataPermissionFilterBuilder, IServiceProvider serviceProvider,
            IMapper mapper) : base(repository, mapper)
        {
            _repository = repository;
            this._trainKnowladgeRepository = trainKnowladgeRepository;
            this._trainPracticeRepository = trainPracticeRepository;
            this._trainPracticeOptionsRepository = trainPracticeOptionsRepository;
            this._trainPracticeSourcewareRepository = trainPracticeSourcewareRepository;
            this._trainPracticeKnowladgeRepository = trainPracticeKnowladgeRepository;
            this._dataPermissionFilterBuilder = dataPermissionFilterBuilder;
            this._serviceProvider = serviceProvider;
            this._mapper = mapper;
        }

        #region public method
        public async Task<TrainTemplateDto> GetAsync(EntityQueryRequest entityQueryRequest)
        {

            var trainTemplate = await _repository.GetAsync(entityQueryRequest.Id);

            var trainTemplateDto = _mapper.Map<TrainTemplateDto>(trainTemplate);

            trainTemplateDto.TrainKnowladges = await GetKnowladges(entityQueryRequest);

            trainTemplateDto.TrainPracticeDtos = await GetTrainPractices(entityQueryRequest);

            return trainTemplateDto;
        }

        public async Task<(int, List<TrainTemplateViewResponse>)> GetListAsync(TrainTemplatePagedRequest queryRequest)
        {
            SqlSugar.ISugarQueryable<TrainTemplateViewResponse> queryResult = QueryRequest(queryRequest);

            var total = await queryResult.CountAsync();
            var items = await queryResult.ToListAsync();

            return (total, items);
        }

        public async Task<PageViewResponse<TrainTemplateViewResponse>> GetPagedListAsync(TrainTemplatePagedRequest queryRequest)
        {
            SqlSugar.ISugarQueryable<TrainTemplateViewResponse> queryResult = QueryRequest(queryRequest);

            var total = await queryResult.CountAsync();
            var items = await queryResult.ToPageListAsync(queryRequest.PageIndex, queryRequest.PageSize);

            return new TrainTemplatePageViewResponse
            {
                Items = items,
                Pagination = new Pagination(queryRequest.PageIndex, queryRequest.PageSize, total)
            };

        }

        public override async Task<string> AddAsync(AddTrainTemplateDto actionRequest, CancellationToken cancellationToken)
        {
            var id = await base.AddAsync(actionRequest, cancellationToken);

            ResolveTemplateId(actionRequest,id);

            await AddTrainKnowladgesAsync(actionRequest, cancellationToken);

            await AddTrainPracticesAsync(actionRequest, cancellationToken);

            await AddTrainPracticeOptionsAsync(actionRequest, cancellationToken);

            await AddTrainPracticeKnowladgeAsync(actionRequest, cancellationToken);

            await AddTrainPracticeSourcewareAsync(actionRequest, cancellationToken);

            return id;

        }
        public override async Task UpdateAsync(UpdateTrainTemplateDto actionRequest, CancellationToken cancellationToken)
        {
            await base.UpdateAsync(actionRequest, cancellationToken);

            await ModifyTrainKnowladgeAsync(actionRequest, cancellationToken);

            await ModifyTrainPracticesAsync(actionRequest, cancellationToken);

            await ModifyTrainPracticeOptionsAsync(actionRequest, cancellationToken);

            await ModifyTrainPracticeKnowladgeAsync(actionRequest, cancellationToken);

            await ModifyTrainPracticeSourcewareAsync(actionRequest, cancellationToken);
        }


        public override async Task DeleteAsync(EntityQueryRequest entityQueryRequest, CancellationToken cancellationToken)
        {
            await base.DeleteAsync(entityQueryRequest, cancellationToken);

            var tmpEntityQueryRequest = ExpressionableUtility.CreateExpression<TrainKnowladge>()
               .AndIF(entityQueryRequest.Id.IsNotNullOrEmpty(), x => x.TrainTemplateId == entityQueryRequest.Id)
               .AndIF(entityQueryRequest.Ids.IsNotNull(), x => entityQueryRequest.Ids.Contains(x.TrainTemplateId))
               .ToEntityQueryRequest<TrainKnowladge>();

            await DeleteTrainKnowladgesAsync(tmpEntityQueryRequest,cancellationToken);

            tmpEntityQueryRequest = ExpressionableUtility.CreateExpression<TrainPractice>()
               .AndIF(entityQueryRequest.Id.IsNotNullOrEmpty(), x => x.TrainTemplateId == entityQueryRequest.Id)
               .AndIF(entityQueryRequest.Ids.IsNotNull(), x => entityQueryRequest.Ids.Contains(x.TrainTemplateId))
               .ToEntityQueryRequest<TrainPractice>();

           await DeleteTrainPracticeAsync(tmpEntityQueryRequest,cancellationToken);
        }
        #endregion

        #region private method
        private async Task<List<TrainKnowladgeDto>> GetKnowladges(EntityQueryRequest entityQueryRequest)
        {
            var trainKnowladgeTable =  _trainKnowladgeRepository.Queryable().Where(x => x.TrainTemplateId == entityQueryRequest.Id);
            var knowledgeTable = new ExamRepository<KnowledgeBase.Knowledge>(_repository.UOW,_dataPermissionFilterBuilder,_serviceProvider).Queryable();

            var queseryResult = trainKnowladgeTable.InnerJoin(knowledgeTable, (t, k) => t.KnowladgeId == k.Id).Select((t, k) => new TrainKnowladgeDto
            {
                Id = t.Id,
                KnowladgeId = t.KnowladgeId,
                Title = k.Title
            });

            return await queseryResult.ToListAsync();

        }

        private async Task ModifyTrainPracticeOptionsAsync(UpdateTrainTemplateDto actionRequest, CancellationToken cancellationToken)
        {
            if (actionRequest.TrainPracticeDtos == null) return;

            var questionIds = actionRequest.TrainPracticeDtos.Where(x => x.QuestionType.CheckSelectType()).Select(x => x.QuestionId).ToList();

            var entityQueryRequest = new EntityQueryRequest
            {
                Expression = ExpressionableUtility.CreateExpression<TrainPracticeOptions>()
                             .AndIF(questionIds.IsNotNull(), x => questionIds.Contains(x.QuestionId)).ToExpression()
            };

            await DeleteTrainPracticeOptionsAsync(entityQueryRequest, cancellationToken);

            await AddTrainPracticeOptionsAsync(actionRequest, cancellationToken);
        }

        private async Task DeleteTrainPracticeOptionsAsync(EntityQueryRequest entityQueryRequest, CancellationToken cancellationToken)
        {
            await _trainPracticeOptionsRepository.DeleteWithValidateAsync(entityQueryRequest, cancellationToken);
        }



        private async Task AddTrainPracticeSourcewareAsync(AddTrainTemplateDto actionRequest, CancellationToken cancellationToken)
        {
            var questionSourcewareRepository = new ExamRepository<QuestionSourceware>(_repository.UOW, _dataPermissionFilterBuilder, _serviceProvider);

            var quesitonIds = actionRequest.TrainPracticeDtos.Select(x => x.QuestionId);

            var questionSourceware = await questionSourcewareRepository.Queryable().Where(x => quesitonIds.Contains(x.QuestionId)).ToListAsync();

            var trainPracticeSourcewares = new List<TrainPracticeSourceware>();

            actionRequest.TrainPracticeDtos.Where(x => x.QuestionType.CheckSelectType()).ToList().ForEach(x =>
            {
                var options = questionSourceware.Where(n => x.QuestionId == n.QuestionId).ToList();

                if (options != null)
                {
                    options.ForEach(item =>
                    {
                        var trainPracticeSourceware = _mapper.Map<QuestionSourceware, TrainPracticeSourceware>(item);

                        trainPracticeSourceware.ToInsert();

                        trainPracticeSourcewares.Add(trainPracticeSourceware);
                    });
                }

            });

            await _trainPracticeSourcewareRepository.AddWithValidateAsync(trainPracticeSourcewares, cancellationToken);
        }

        private async Task AddTrainPracticeKnowladgeAsync(AddTrainTemplateDto actionRequest, CancellationToken cancellationToken)
        {
            var questionKnowladgeRepository = new ExamRepository<QuestionKnowladge>(_repository.UOW, _dataPermissionFilterBuilder, _serviceProvider);

            var quesitonIds = actionRequest.TrainPracticeDtos.Select(x => x.QuestionId);

            var questionKnowladge = await questionKnowladgeRepository.Queryable().Where(x => quesitonIds.Contains(x.QuestionId)).ToListAsync();

            var trainPracticeKnowladges = new List<TrainPracticeKnowladge>();

            actionRequest.TrainPracticeDtos.Where(x => x.QuestionType.CheckSelectType()).ToList().ForEach(x =>
            {
                var options = questionKnowladge.Where(n => x.QuestionId == n.QuestionId).ToList();

                if (options != null)
                {
                    options.ForEach(item =>
                    {
                        var trainPracticeKnowladge = _mapper.Map<QuestionKnowladge, TrainPracticeKnowladge>(item);

                        trainPracticeKnowladge.ToInsert();

                        trainPracticeKnowladges.Add(trainPracticeKnowladge);
                    });
                }

            });

            await _trainPracticeKnowladgeRepository.AddWithValidateAsync(trainPracticeKnowladges, cancellationToken);
        }

        private async Task ModifyTrainPracticeSourcewareAsync(UpdateTrainTemplateDto actionRequest, CancellationToken cancellationToken)
        {
            if (actionRequest.TrainPracticeDtos == null) return;

            var questionIds = actionRequest.TrainPracticeDtos.Where(x => x.QuestionType.CheckSelectType()).Select(x => x.QuestionId).ToList();

            var entityQueryRequest = new EntityQueryRequest
            {
                Expression = ExpressionableUtility.CreateExpression<TrainPracticeSourceware>()
                             .AndIF(questionIds.IsNotNull(), x => questionIds.Contains(x.QuestionId)).ToExpression()
            };

            await DeleteTrainPracticeSourcewareAsync(entityQueryRequest, cancellationToken);

            await AddTrainPracticeSourcewareAsync(actionRequest, cancellationToken);
        }

        private async Task DeleteTrainPracticeSourcewareAsync(EntityQueryRequest entityQueryRequest, CancellationToken cancellationToken)
        {
            await _trainPracticeSourcewareRepository.DeleteWithValidateAsync(entityQueryRequest, cancellationToken);
        }

        private async Task ModifyTrainPracticeKnowladgeAsync(UpdateTrainTemplateDto actionRequest, CancellationToken cancellationToken)
        {
            if (actionRequest.TrainPracticeDtos == null) return;

            var questionIds = actionRequest.TrainPracticeDtos.Where(x => x.QuestionType.CheckSelectType()).Select(x => x.QuestionId).ToList();

            var entityQueryRequest = new EntityQueryRequest
            {
                Expression = ExpressionableUtility.CreateExpression<TrainPracticeKnowladge>()
                             .AndIF(questionIds.IsNotNull(), x => questionIds.Contains(x.QuestionId)).ToExpression()
            };

            await DeleteTrainPracticeKnowladgeAsync(entityQueryRequest, cancellationToken);

            await AddTrainPracticeKnowladgeAsync(actionRequest, cancellationToken);
        }

        private async Task DeleteTrainPracticeKnowladgeAsync(EntityQueryRequest entityQueryRequest, CancellationToken cancellationToken)
        {
            await _trainPracticeKnowladgeRepository.DeleteWithValidateAsync(entityQueryRequest, cancellationToken);
        }

        private async Task AddTrainPracticeOptionsAsync(AddTrainTemplateDto actionRequest, CancellationToken cancellationToken)
        {
            var questionOptionRepository = new ExamRepository<QuestionOptions>(_repository.UOW, _dataPermissionFilterBuilder, _serviceProvider);

            var quesitonIds = actionRequest.TrainPracticeDtos.Select(x => x.QuestionId);

            var questionOptions = await questionOptionRepository.Queryable().Where(x => quesitonIds.Contains(x.QuestionId)).ToListAsync();

            var trainPracticeOptions = new List<TrainPracticeOptions>();

            actionRequest.TrainPracticeDtos.Where(x => x.QuestionType.CheckSelectType()).ToList().ForEach(x =>
            {
                var options = questionOptions.Where(n => x.QuestionId == n.QuestionId).ToList();

                if (options != null)
                {
                    options.ForEach(item =>
                    {
                        var trainPracticeOption = _mapper.Map<QuestionOptions, TrainPracticeOptions>(item);

                        trainPracticeOption.ToInsert();

                        trainPracticeOptions.Add(trainPracticeOption);
                    });
                }

            });

            await _trainPracticeOptionsRepository.AddWithValidateAsync(trainPracticeOptions, cancellationToken);
        }

        private void ResolveTemplateId(AddTrainTemplateDto actionRequest, string id)
        {
            if (actionRequest.TrainKnowladges != null)
            {
                actionRequest.TrainKnowladges.ForEach(x => x.TrainTemplateId = id);
            }

            if (actionRequest.TrainPracticeDtos != null)
            {
                actionRequest.TrainPracticeDtos.ForEach(x => x.TrainTemplateId = id);
            }

        }

        private async Task<List<TrainPracticeDto>> GetTrainPractices(EntityQueryRequest entityQueryRequest)
        {
            var questionTable = new ExamRepository<Question>(_repository.UOW, _dataPermissionFilterBuilder, _serviceProvider).Queryable();
            var trainPracticeTable = _trainPracticeRepository.Queryable().Where(x => x.TrainTemplateId == entityQueryRequest.Id);

            var trainPractices = trainPracticeTable.InnerJoin(questionTable, (t, q) => t.QuestionId == q.Id).Select((t, q) => new TrainPracticeDto
            {
                Id = t.Id,
                QuestionId =t.QuestionId,
                Title = q.Title
            });

            return await trainPractices.ToListAsync();
        }

        private SqlSugar.ISugarQueryable<TrainTemplateViewResponse> QueryRequest(TrainTemplatePagedRequest queryRequest)
        {
            var expression = queryRequest.GetExpression();
            var queryable = _repository.Queryable().Where(expression);

            var queryResult = queryable.Select(m => new TrainTemplateViewResponse
            {
                Id = m.Id,
                Code = m.Code,
                Name = m.Name,
                IsContainsPractice = m.IsContainsPractice,
                SortIndex = m.SortIndex,
                Status = m.Status,
                CreationTime = m.CreationTime,
                CreatorName = m.CreatorName,
                CreatorOrgName = m.CreatorOrgName
            });
            return queryResult;
        }

        private async Task AddTrainPracticesAsync(AddTrainTemplateDto actionRequest, CancellationToken cancellationToken)
        {
            if (actionRequest.TrainKnowladges != null)
            {
                var trainKnowladgeDtos = actionRequest.TrainKnowladges.Where(x => x.OperationStatus == EEOperationStatus.Add).ToList();

                var trainKnowladges = _mapper.Map<List<TrainKnowladge>>(trainKnowladgeDtos);

                //TODO:排序
                var sortIndex = 0;
                trainKnowladges.ForEach(x =>
                {
                    x.SortIndex = sortIndex + 1;
                });

                trainKnowladges.ToInsert();

                await _trainKnowladgeRepository.AddWithValidateAsync(trainKnowladges, cancellationToken);
            }
        }

        private async Task AddTrainKnowladgesAsync(AddTrainTemplateDto actionRequest, CancellationToken cancellationToken)
        {
            if (actionRequest.TrainPracticeDtos != null)
            {
                var trainPracticeDtos = actionRequest.TrainPracticeDtos.Where(x => x.OperationStatus == EEOperationStatus.Add).ToList();

                var trainPractices = _mapper.Map<List<TrainPractice>>(trainPracticeDtos);

                trainPractices.ToInsert();

                await _trainPracticeRepository.AddWithValidateAsync(trainPractices, cancellationToken);
            }
        }

        private async Task ModifyTrainPracticesAsync(UpdateTrainTemplateDto actionRequest, CancellationToken cancellationToken)
        {
            await AddTrainPracticesAsync(actionRequest, cancellationToken);

            await UpdateTrainPracticesAsync(actionRequest, cancellationToken);

            if (actionRequest.TrainPracticeDtos != null)
            {
                var ids = actionRequest.TrainPracticeDtos.Where(x => x.OperationStatus == EEOperationStatus.Delete).Select(x => x.Id).ToList();

                var entityQueryRequest = new EntityQueryRequest
                {
                    Expression = ExpressionableUtility.CreateExpression<TrainPractice>()
                    .AndIF(ids.IsNotEmpty(), x => ids.Contains(x.Id))
                    .ToExpression()
                };

                await DeleteTrainPracticeAsync(entityQueryRequest,cancellationToken);
            }
           
            
        }

        private async Task DeleteTrainPracticeAsync(EntityQueryRequest entityQueryRequest, CancellationToken cancellationToken)
        {
            await _trainPracticeRepository.DeleteWithValidateAsync(entityQueryRequest, cancellationToken);
        }

        private async Task UpdateTrainPracticesAsync(UpdateTrainTemplateDto actionRequest, CancellationToken cancellationToken)
        {
            if (actionRequest.TrainPracticeDtos == null) return;

            var trainPracticeDtos = actionRequest.TrainPracticeDtos.Where(x => x.OperationStatus == EEOperationStatus.Update).ToList();

            var ids = trainPracticeDtos.Select(x => x.Id);

            var trainPractices = await _trainPracticeRepository.Queryable().Where(x => ids.Contains(x.Id)).ToListAsync();

            trainPractices = _mapper.Map<List<UpdateTrainPracticeDto>, List<TrainPractice>>(trainPracticeDtos, trainPractices);

            trainPractices.ToUpdate();

            await _trainPracticeRepository.UpdateWithValidateAsync(trainPractices, cancellationToken);
        }

        private async Task ModifyTrainKnowladgeAsync(UpdateTrainTemplateDto actionRequest, CancellationToken cancellationToken)
        {
            await AddTrainKnowladgesAsync(actionRequest, cancellationToken);

            await UpdateTrainKnowladgesAsync(actionRequest, cancellationToken);

            if (actionRequest.TrainKnowladges != null)
            {
                var ids = actionRequest.TrainKnowladges.Where(x => x.OperationStatus == EEOperationStatus.Delete).Select(x => x.Id).ToList();

                var entityQueryRequest = new EntityQueryRequest
                {
                    Expression = ExpressionableUtility.CreateExpression<TrainKnowladge>()
                    .AndIF(ids.IsNotEmpty(), x => ids.Contains(x.Id))
                    .ToExpression()
                };

                await DeleteTrainKnowladgesAsync(entityQueryRequest, cancellationToken);
            }
        }

        private async Task DeleteTrainKnowladgesAsync(EntityQueryRequest entityQueryRequest, CancellationToken cancellationToken)
        {
            await _trainKnowladgeRepository.DeleteWithValidateAsync(entityQueryRequest, cancellationToken);
        }

        private async Task UpdateTrainKnowladgesAsync(UpdateTrainTemplateDto actionRequest, CancellationToken cancellationToken)
        {
            if (actionRequest.TrainKnowladges == null) return;

            var trainKnowladgeDtos = actionRequest.TrainKnowladges.Where(x => x.OperationStatus == EEOperationStatus.Update).ToList();

            var ids = trainKnowladgeDtos.Select(x => x.Id);

            var trainKnowladges = await _trainKnowladgeRepository.Queryable().Where(x => ids.Contains(x.Id)).ToListAsync();

            trainKnowladges = _mapper.Map<List<UpdateTrainKnowladgeDto>, List<TrainKnowladge>>(trainKnowladgeDtos, trainKnowladges);

            trainKnowladges.ToUpdate();

            await _trainKnowladgeRepository.UpdateWithValidateAsync(trainKnowladges, cancellationToken);
        }
        #endregion
    }
}
